/* Pxys - An IRC ProxyScan, main.c
 * 
 * Copyright (C) 2001, 2002 by Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * 
 * Notes:  mp_  : proxy module
 *         ms_  : server module
 *         m_   : general module
 *
 * This program was first of all coded for freeBSD, and dedicated to
 * the Undernet IRC network.
 *
 *
 * $Id: main.c,v 1.15 2002/12/04 15:50:21 mbuna Exp $
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <errno.h>

#if HAVE_PTHREAD_H
#include <pthread.h>
#endif
#if HAVE_FCNTL_H
#include <fcntl.h>
#endif
#if HAVE_SYS_UIO_H
#include <sys/uio.h>
#endif
#if HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#include "conf.h"
#include "debug.h"
#include "tokenizer.h" /* tokenizer_init() */
#include "typedefs.h"

#include "ms_globals.h"
#include "ms_msgout.h" /* msgout_init() */
#include "ms_myclient.h"
#include "ms_network.h"
#include "ms_parser.h"
#include "ms_socket.h"

#include "m_array.h"
#include "m_cache.h"
#include "m_clientq.h"
#include "m_glineq.h"
#include "m_log.h"
#include "m_opercmds.h"
#include "m_proxycache.h"
#include "m_signal.h"
#include "m_scanner.h"
#include "m_stats.h"
#include "m_shutdown.h"
#include "m_timers.h"


/* Should be defined in sys/stat.h */
#ifndef S_IREAD
#define S_IREAD 0000400
#endif
#ifndef S_IWRITE
#define S_IWRITE 0000200
#endif

/* Prototype */
void pxys_exit(int x);

long StartTime;

/*
static pthread_once_t init_once = PTHREAD_ONCE_INIT;
static pthread_key_t key;
static int init_status;


static void _serv_init_global()
{
	init_status = pthread_key_create(&key, free);
}
*/


static void setup_signals()
{
   /* Signal handling */
   if(signal(SIGTERM, sig_term) == SIG_IGN)
      signal(SIGTERM, SIG_IGN);
   if(signal(SIGINT, sig_int) == SIG_IGN)
      signal(SIGINT, SIG_IGN);
   if(signal(SIGHUP, sig_hup) == SIG_IGN)
      signal(SIGHUP, SIG_IGN);
   
   if(signal(SIGUSR1, sig_usr1) == SIG_IGN)
      signal(SIGUSR1, SIG_IGN);
   
   (void)signal(SIGPIPE, SIG_IGN);
   (void)signal(SIGURG, SIG_IGN);
   
   /* Disabled alarm engine, that was a bad idea, thanks to ripper_'s advices!
    * Added/moved timers in ms_socket.c (using select as a base for all timers)
    *
   if(signal(SIGALRM, mp_alarm) == SIG_IGN)
      signal(SIGALRM, mp_alarm);
    */
   
   (void)signal(SIGALRM, SIG_IGN);
}


static void save_pid()
{
  char temp[8];
  char pidfile[] = "pxys.pid";
  int pidfile_fd;
  
  pidfile_fd = open(pidfile, O_WRONLY | O_CREAT | O_TRUNC, S_IREAD | S_IWRITE);
  
  if(pidfile_fd < 0)
    fprintf(stderr, "Error opening pid file %s: %s\n", pidfile, strerror(errno));
  else {
    memset(temp, 0, sizeof(temp));
    snprintf(temp, sizeof(temp), "%5d\n", getpid());
    write(pidfile_fd, temp, strlen(temp));
    close(pidfile_fd);
  }
}

static void usage()
{
   printf("*** Usage: pxys [-fv]\n");
   printf("***   -f     do not fork\n");
   printf("***   -o     connect once, dont try to reconnect\n");
   printf("***   -v     print pxys version\n");
   exit(0);
}


int
main(int argc, char **argv)
{
   unsigned char dofork = 1, once = 0;
   char ch;
   
   time(&StartTime);
   srand((unsigned int) StartTime);
   
   /* options in command line
    *  -f : dont fork
    *  -v : print version
    */
   while ((ch = getopt(argc, argv, "fov")) != -1)
      switch(ch) {
      case 'f':
         dofork = 0;
         break;
      case 'o':
         once = 1;
         break;
      case 'v':
         printf("\n*** pxys " VERSION "\n\n");
         exit(0);
         
      case '?':
      default:
         usage();
   }
   
   /*
   pthread_once(&init_once, _serv_init_global);
   */
   
   save_pid();
   setup_signals();
   
   /* Read pxys.conf */
   if(!conf_load()) {
      fprintf(stderr, "pxys: configuration check failed.\n");
      exit(1);
   } else
      printf("pxys: configuration file loaded.\n");
   
   if (!conf_resolv()) {
      fprintf(stderr, "pxys: socks4: no valid target host found, aborting.\n");
      exit(1);
   }
   
   printf("\n");
   
   log_init();
   printf("Syncing log files... ");
   log_rotate_sync();
   printf("done.\n");
   
   log_slacken_init();
   
   m_timers_init();
   
   tokenizer_init();
   clientq_init();
   proxycache_init();
   msgout_init();
   parser_init();
   myclient_init();
   glineq_init();
   network_init();
   array_init();
   cache_init();
   printf("Loading cache... ");
   fflush(stdout);
   printf("%d IPs\n", cache_load());
   stats_init();
   
   if (dofork) {
      if (fork())
         exit(0);
      /* child's pid */
      save_pid();
   }
   
   scanner_init();
   
   sleep(1);
   
   
   log_error("Starting");
   
   /* This endless loop handle reconnection... */
   irc_start();
      
   proxycache_syncfile();
   
   pxys_exit(0);
   return 0;
}

void
pxys_exit(int x)
{
   proxycache_close();
   proxycache_free();
   cache_free();
   clientq_destroy();
   network_free();
   tokenizer_free();
   glineq_destroy();
   msgout_free();
   parser_destroy();
   myclient_destroy();
   lastcomm_free();
   lastlog_free();
   stats_free();
   scanner_destroy();
   array_destroy();
   log_close();
   log_slacken_destroy();
   conf_delete();
   exit(x);
}


/* main.c */
